@@ -13,15 +13,16 @@ import sys
 import time
 from collections import deque
 from operator import itemgetter
-from iframes import IFramer
 from new_reader import reader
+from iframes import IFramer
 from threefive import Cue
 import threefive.stream as strm
+from m3ufu import M3uFu
 
 
 MAJOR = "0"
-MINOR = "1"
-MAINTAINENCE = "87"
+MINOR = "2"
+MAINTAINENCE = "03"
 
 
 def version():
@@ -50,18 +51,19 @@ class X9K3(strm.Stream):
         self.in_stream = tsdata
         self.active_segment = io.BytesIO()
         self.iframer = IFramer(shush=True)
-        self.window = SlidingWindow(5)
         self.scte35 = SCTE35()
         self.sidecar = deque()
         self.timer = Timer()
         self.m3u8 = "index.m3u8"
+        self.window = SlidingWindow()
+        self.segnum = None
         self.args = argue()
         self.apply_args()
         self.started = None
         self.next_start = None
-        self.segnum = 0
         self.media_seq = 0
         self.discontinuity_sequence = 0
+        self.first_segment = True
 
     def _args_version(self):
         if self.args.version:
@@ -69,11 +71,11 @@ class X9K3(strm.Stream):
             sys.exit()
 
     def _args_input(self):
+        self.in_stream = self.args.input
         if self._tsdata is not None:
             self.args.input = self._tsdata
         else:
             self._tsdata = self.args.input
-        self.in_stream = self.args.input
 
     def _args_hls_tag(self):
         tag_map = {
@@ -89,19 +91,34 @@ class X9K3(strm.Stream):
     def _args_output_dir(self):
         if not os.path.isdir(self.args.output_dir):
             os.mkdir(self.args.output_dir)
-        # self.m3u8 = self.mk_uri(self.args.output_dir, "index.m3u8")
 
+    def _chk_flags(self, flags):
+        if flags:
+            return True in flags
+        return None
 
     def _args_flags(self):
-        if self.args.program_date_time or self.args.delete or self.args.replay:
+        """
+         I really expected to do more here.
+        """
+        flags = deque([self.args.program_date_time, self.args.delete, self.args.replay])
+        if self._chk_flags(flags):
             self.args.live = True
-            if self.args.delete or self.args.replay:
-                self.window.delete = True
+        flags.popleft()  # pop self.args.program_date_time
+        if self._chk_flags(flags):
+            self.window.delete = True
+        flags.popleft()  # pop self.args.delete
+
+        flags.popleft()  # pop self.args.replay
 
     def _args_window_size(self):
         if self.args.live:
             self.window.size = self.args.window_size
 
+    def _args_continue_m3u8(self):
+        if self.args.continue_m3u8:
+            self.continue_m3u8()
+
     def apply_args(self):
         """
         _apply_args  uses command line args
@@ -113,10 +130,73 @@ class X9K3(strm.Stream):
         self._args_output_dir()
         self._args_flags()
         self._args_window_size()
+        self._args_continue_m3u8()
 
         if isinstance(self._tsdata, str):
             self._tsdata = reader(self._tsdata)
 
+    def _reload_chunk(self,segment):
+        tmp_segnum = int(segment.relative_uri.split("seg")[1].split(".")[0])
+        chunk = Chunk(segment.relative_uri,segment.media,tmp_segnum)
+        for this in ["#EXT-X-X9K3-VERSION", "#EXT-X-ENDLIST"]:
+            if this in segment.tags:
+                segment.tags.pop(this)
+        chunk.tags = segment.tags
+        self.window.push_pane(chunk)
+
+    def reload_m3u8(self):
+        """
+        m3u8_reload is called when the continue_m3u8 option is set.
+        The index.m3u8 file is copied to tmp.m3u8 and a "#EXT-X-ENDLIST" tag
+        is appended so that m3ufu doesn't keep trying to reload it as a live stream.
+        An M3uFu instance parses tmp.m3u8 and loads the data into
+        the SlidingWindow, X9K3.window.
+        """
+        m3 = M3uFu()
+        tmp_name = "tmp.m3u8"
+        with open(tmp_name,'w') as tmp_m3u8:
+            with open(self.m3u8uri(),"r") as m3u8:
+                tmp_m3u8.write("\n".join(m3u8.readlines()))
+                tmp_m3u8.write("\n#EXT-X-ENDLIST\n")
+        m3.m3u8 = tmp_name
+        m3.decode()
+        segments =list(m3.segments)
+        for segment in segments:
+             self._reload_chunk(segment)
+        if self.args.live:
+            self._discontinuity_seq_plus_one()
+            self.window.slide_panes()
+        os.unlink(tmp_name)
+
+
+    def continue_m3u8(self):
+        """
+        continue_m3u8 reads self.discontinuity_sequence
+        and self.segnum from an existing index.m3u8
+        when the self.args.continue_m3u8 flag is set.
+        """
+        try:
+            with open(self.m3u8uri(), "r") as manifest:
+                lines = manifest.readlines()
+                match_this = "#EXT-X-DISCONTINUITY-SEQUENCE:"
+                disco_seq = [line for line in lines if match_this in line]
+                for d in disco_seq:
+                    seq = int(d.split(":")[1])
+                    self.discontinuity_sequence = seq
+                segment_list = [line for line in lines if not line.startswith("#")]
+                self.segnum = int(segment_list[-1].split("seg")[1].split(".")[0]) + 1
+            self.reload_m3u8()
+            print(f"Continuing {self.m3u8uri()} @ segment number {self.segnum}")
+        except:
+            pass
+
+
+    def m3u8uri(self):
+        """
+        m3u8uri return full path to the output index.m3u8
+        """
+        return self.mk_uri(self.args.output_dir, self.m3u8)
+
     @staticmethod
     def mk_uri(head, tail):
         """
@@ -153,6 +233,13 @@ class X9K3(strm.Stream):
             ]
         )
 
+    def add_discontinuity(self, chunk):
+        """
+        add_discontinuity adds a discontinuity tag.
+        """
+        if not self.args.no_discontinuity:
+            chunk.add_tag("#EXT-X-DISCONTINUITY", None)
+
     def _add_cue_tag(self, chunk):
         """
         _add_cue_tag adds SCTE-35 tags,
@@ -166,7 +253,7 @@ class X9K3(strm.Stream):
         tag = self.scte35.mk_cue_tag()
         if tag:
             if self.scte35.cue_state in ["OUT", "IN"]:
-                chunk.add_tag("#EXT-X-DISCONTINUITY", None)
+                self.add_discontinuity(chunk)
             kay = tag
             vee = None
             if ":" in tag:
@@ -181,8 +268,15 @@ class X9K3(strm.Stream):
             chunk.add_tag("#EXT-X-PROGRAM-DATE-TIME", f"{iso8601}")
 
     def _chk_live(self, seg_time):
+        """
+        _chk_live
+
+            * slides the sliding window
+            * throttles to simulate live stream
+            * increments discontinuity sequence
+        """
         if self.args.live:
-            self.window.pop_pane()
+            self.window.popleft_pane()
             self.timer.throttle(seg_time)
             self._discontinuity_seq_plus_one()
 
@@ -196,13 +290,27 @@ class X9K3(strm.Stream):
         two = f"end: {self.next_start:.6f}   duration: {seg_time:.6f}"
         print(f"{one}{two}", file=sys.stderr)
 
+    def now(self):
+        """
+        now returns the current pts
+        for the first program available.
+        """
+        return self.as_90k(list(self.maps.prgm_pts.items())[0][1])
+
     def _write_segment(self):
+        if self.segnum is None:
+            self.segnum = 0
         seg_file = f"seg{self.segnum}.ts"
         seg_name = self.mk_uri(self.args.output_dir, seg_file)
-        seg_time = round(self.next_start - self.started, 6)
+        seg_time = round(self.now() - self.started, 6)
         with open(seg_name, "wb") as seg:
             seg.write(self.active_segment.getbuffer())
-        chunk = Chunk(seg_name, self.segnum)
+        if seg_time <= 0:
+            return
+        chunk = Chunk(seg_file, seg_name, self.segnum)
+        if self.first_segment:
+            if self.args.replay or self.args.continue_m3u8:
+                self.add_discontinuity(chunk)
         self._mk_chunk_tags(chunk, seg_time)
         self.window.push_pane(chunk)
         self._write_m3u8()
@@ -213,15 +321,20 @@ class X9K3(strm.Stream):
         self.scte35.chk_cue_state()
         self._chk_live(seg_time)
 
+
+    def _clear_endlist(self,lines):
+        return [line for line in lines if "ENDLIST" not in line]
+
     def _write_m3u8(self):
+        """
+        _write_m3u8 writes the index.m3u8
+        """
         self.media_seq = self.window.panes[0].num
-        m3u8uri = self.mk_uri(self.args.output_dir, self.m3u8)
-        with open(m3u8uri, "w+") as m3u8:
+        with open(self.m3u8uri(), "w+") as m3u8:
             m3u8.write(self._header())
             m3u8.write(self.window.all_panes())
             self.segnum += 1
-            if not self.args.live:
-                m3u8.write("#EXT-X-ENDLIST")
+            self.first_segment = False
         self.active_segment = io.BytesIO()
 
     def _load_sidecar(self, pid):
@@ -239,17 +352,21 @@ class X9K3(strm.Stream):
                         insert_pts = float(insert_pts)
                         if insert_pts == 0.0 and self.args.live:
                             insert_pts = self.next_start
-                        if insert_pts >= self.pid2pts(pid):
-                            if [insert_pts, cue] not in self.sidecar:
-                                self.sidecar.append([insert_pts, cue])
-                                self.sidecar = deque(
-                                    sorted(self.sidecar, key=itemgetter(0))
-                                )
+                        if insert_pts > self.pid2pts(pid):
+                            self.add2sidecar(insert_pts, cue)
                 sidefile.close()
-            if self.args.live:
+            if self.args.live and not self.args.replay:
                 with open(self.args.sidecar_file, "w") as scf:
                     scf.close()
 
+    def add2sidecar(self, insert_pts, cue):
+        """
+        add2sidecar add insert_pts,cue to the deque
+        """
+        if [insert_pts, cue] not in self.sidecar:
+            self.sidecar.append([insert_pts, cue])
+            self.sidecar = deque(sorted(self.sidecar, key=itemgetter(0)))
+
     def _chk_sidecar_cues(self, pid):
         """
         _chk_sidecar_cues checks the insert pts time
@@ -261,7 +378,7 @@ class X9K3(strm.Stream):
                 self.scte35.cue_time = float(raw[0])
                 self.scte35.cue = Cue(raw[1])
                 self.scte35.cue.decode()
-                # self.scte35.cue.show()
+                self.scte35.cue.show()
                 self._chk_cue_time(pid)
 
     def _discontinuity_seq_plus_one(self):
@@ -290,13 +407,13 @@ class X9K3(strm.Stream):
         """
         if self.scte35.cue_time:
             if now >= self.scte35.cue_time:
-                self.next_start = self.scte35.cue_time
+                self.next_start = self.now()
                 self._write_segment()
                 self.scte35.cue_time = None
                 self.scte35.mk_cue_state()
                 return
         if now >= self.started + self.args.time:
-            self.next_start = now
+            self.next_start = self.now()
             self._write_segment()
 
     def _chk_cue_time(self, pid):
@@ -306,50 +423,91 @@ class X9K3(strm.Stream):
         the cue is received.
         """
         if self.scte35.cue:
-            pts_adjust = self.scte35.cue.info_section.pts_adjustment
-            if "pts_time" in self.scte35.cue.command.get():
-                self.scte35.cue_time = self.scte35.cue.command.pts_time + pts_adjust
-            else:
-                self.scte35.cue_time = self.pid2pts(pid) + pts_adjust
+            self.scte35.cue_time = self.adjusted_pts(self.scte35.cue, pid)
+
+    def adjusted_pts(self, cue, pid):
+        """
+        adjusted_pts = (pts_time + pts_adjustment) % self.ROLLOVER
+        """
+        pts = 0
+        if "pts_time" in cue.command.get():
+            pts = cue.command.pts_time
+        else:
+            pts = self.pid2pts(pid)
+        pts_adjust = cue.info_section.pts_adjustment
+        adj_pts = (pts + pts_adjust) % self.as_90k(self.ROLLOVER)
+        return round(adj_pts, 6)
 
     @staticmethod
     def _rai_flag(pkt):
+        """
+        _rai_flag random access indicator flag
+        """
         return pkt[5] & 0x40
 
     def _shulga_mode(self, pkt, now):
         """
-        _shulga_mode iframe detection
+        _shulga_mode is mpeg2 video iframe detection
         """
         if self._rai_flag(pkt):
             self._chk_slice_point(now)
 
     def _parse_scte35(self, pkt, pid):
+        """
+        _parse_scte35 overrides the inherited
+        Stream._parse_scte35 method
+        """
         cue = super()._parse_scte35(pkt, pid)
         if cue:
             cue.decode()
-            # cue.show()
             self.scte35.cue = cue
             self._chk_cue_time(pid)
+            self.add2sidecar(self.adjusted_pts(cue, pid), cue.encode())
         return cue
 
     def _parse(self, pkt):
+        """
+        _parse is run on every packet.
+        """
         super()._parse(pkt)
-
         pkt_pid = self._parse_pid(pkt[1], pkt[2])
         now = self.pid2pts(pkt_pid)
         if not self.started:
             self._start_next_start(pts=now)
         if self._pusi_flag(pkt) and self.started:
-            self._load_sidecar(pkt_pid)
-            self._chk_sidecar_cues(pkt_pid)
             if self.args.shulga:
                 self._shulga_mode(pkt, now)
             else:
                 i_pts = self.iframer.parse(pkt)
                 if i_pts:
                     self._chk_slice_point(i_pts)
+            # Split on non-Iframes for CUE-IN or CUE-OUT
+            if self.scte35.cue_time:
+                self._chk_slice_point(now)
+            self._load_sidecar(pkt_pid)
+            self._chk_sidecar_cues(pkt_pid)
         self.active_segment.write(pkt)
 
+
+    def addendum(self):
+        """
+        addendum post stream parsing related tasks.
+            * writing the last segment
+
+            * sleeping to ensure last segment gets playing
+            when the replay flag or continue_m3u8 flag is set.
+
+            * adding endlist tag
+
+        """
+        buff = self.active_segment.getbuffer()
+        if buff:
+            self._write_segment()
+        if not self.args.live:
+            with open(self.m3u8uri(), "a") as m3u8:
+                m3u8.write("#EXT-X-ENDLIST")
+
+
     def decode(self, func=False):
         """
         decode iterates mpegts packets
@@ -358,31 +516,7 @@ class X9K3(strm.Stream):
         """
         self.timer.start()
         super().decode()
-
-    def loop(self):
-        """
-        loop  loops a video in the hls manifest.
-        sliding window and throttled to simulate live playback,
-        segments are deleted when they fall out the sliding window.
-        """
-        self.decode()
-        self._reset_stream()
-        with open(self.m3u8, "w+") as m3u8:
-            m3u8.write("#EXT-X-DISCONTINUITY")
-        self._tsdata = reader(self.in_stream)
-        return True
-
-    def run(self):
-        """
-        run calls replay() if replay is set
-        or else it calls decode()
-        """
-        self.apply_args()
-        if self.args.replay:
-            while True:
-                self.loop()
-        else:
-            self.decode()
+        self.addendum()
 
 
 class SCTE35:
@@ -399,6 +533,7 @@ class SCTE35:
         self.break_timer = None
         self.break_duration = None
         self.event_id = 1
+        self.seg_type = None
 
     def mk_cue_tag(self):
         """
@@ -486,7 +621,6 @@ class SCTE35:
         if self.cue_state == "OUT":
             fstart = f',START-DATE="{iso8601}"'
             tag = f"{fbase}{fstart}{fdur},SCTE35-OUT={self.cue.encode_as_hex()}"
-            self.event_id += 1
             return tag
 
         if self.cue_state == "IN":
@@ -504,6 +638,8 @@ class SCTE35:
         """
         if cue is None:
             return False
+        if self.cue_state not in ["IN", None]:
+            return False
         cmd = cue.command
         if cmd.command_type == 5:
             if cmd.out_of_network_indicator:
@@ -511,27 +647,15 @@ class SCTE35:
                     self.break_duration = cmd.break_duration
                     return True
 
-        upid_starts = [
-            0x10,
-            0x20,
-            0x22,
-            0x30,
-            0x32,
-            0x34,
-            0x36,
-            0x38,
-            0x3A,
-            0x3C,
-            0x3E,
-            0x44,
-            0x46,
-        ]
+        seg_starts = [0x22, 0x30, 0x32, 0x34, 0x36, 0x44, 0x46]
         if cmd.command_type == 6:
             for dsptr in cue.descriptors:
                 if dsptr.tag == 2:
-                    if dsptr.segmentation_type_id in upid_starts:
+                    if dsptr.segmentation_type_id in seg_starts:
+                        self.seg_type = dsptr.segmentation_type_id + 1
                         if dsptr.segmentation_duration:
                             self.break_duration = dsptr.segmentation_duration
+                            self.cue_state = "OUT"
                             return True
         return False
 
@@ -541,32 +665,21 @@ class SCTE35:
         to see if it is a cue_in event.
         Returns True for a cue_in event.
         """
-        if cue is not None:
-            cmd = cue.command
-            if cmd.command_type == 5:
-                if not cmd.out_of_network_indicator:
-                    return True
-
-            upid_stops = [
-                0x11,
-                0x21,
-                0x21,
-                0x23,
-                0x33,
-                0x35,
-                0x37,
-                0x39,
-                0x3B,
-                0x3D,
-                0x3F,
-                0x45,
-                0x47,
-            ]
-            if cmd.command_type == 6:
-                for dsptr in cue.descriptors:
-                    if dsptr.tag == 2:
-                        if dsptr.segmentation_type_id in upid_stops:
-                            return True
+        if cue is None:
+            return False
+        if self.cue_state not in ["OUT", "CONT"]:
+            return False
+        cmd = cue.command
+        if cmd.command_type == 5:
+            if not cmd.out_of_network_indicator:
+                return True
+        if cmd.command_type == 6:
+            for dsptr in cue.descriptors:
+                if dsptr.tag == 2:
+                    if dsptr.segmentation_type_id == self.seg_type:
+                        self.seg_type = None
+                        self.cue_state = "IN"
+                        return True
         return False
 
 
@@ -577,26 +690,29 @@ class SlidingWindow:
 
     def __init__(self, size=10000):
         self.size = size
-        self.panes = []
+        self.panes = deque()
         self.delete = False
 
-    def pop_pane(self):
+    def popleft_pane(self):
         """
-        pop_pane removes the first item in self.panes
+        popleft_pane removes the first item in self.panes
         """
         if len(self.panes) >= self.size:
             if self.delete:
                 popped = self.panes[0].name
                 print(f"deleting {popped}")
-                os.unlink(popped)
-            self.panes = self.panes[1:]
+                try:
+                    os.unlink(popped)
+                except:
+                    pass
+            return self.panes.popleft()
 
     def push_pane(self, a_pane):
         """
         push appends a_pane to self.panes
         """
         self.panes.append(a_pane)
-        # print([a_pane.name for a_pane in self.panes])
+
 
     def all_panes(self):
         """
@@ -604,14 +720,16 @@ class SlidingWindow:
         """
         return "".join([a_pane.get() for a_pane in self.panes])
 
-    def slide_panes(self, a_pane):
+    def slide_panes(self, a_pane=None):
         """
         slide calls self.push_pane with a_pane
-        and then calls self.pop_pane to trim self.panes
+        and then calls self.popleft_pane to trim self.panes
         as needed.
         """
-        self.push_pane(a_pane)
-        self.pop_pane()
+        if a_pane:
+            self.push_pane(a_pane)
+        while len(self.panes) >= self.size:
+            self.popleft_pane()
 
 
 class Timer:
@@ -660,7 +778,7 @@ class Timer:
         self.stop(end)
         diff = round(seg_time - self.lap_time, 2)
         if diff > 0:
-            # print(f"throttling {diff}")#,file=sys.stderr, end='\r')
+            print(f"throttling {diff}")  # ,file=sys.stderr, end='\r')
             time.sleep(diff)
         self.start(begin)
 
@@ -671,8 +789,9 @@ class Chunk:
     for a segment.
     """
 
-    def __init__(self, name, num):
+    def __init__(self, file, name, num):
         self.tags = {}
+        self.file = file
         self.name = name
         self.num = num
 
@@ -686,7 +805,7 @@ class Chunk:
                 this.append(kay)
             else:
                 this.append(f"{kay}:{vee}")
-        this.append(self.name)
+        this.append(self.file)
         this.append("")
         this = "\n".join(this)
         return this
@@ -699,7 +818,6 @@ class Chunk:
 
 
 def argue():
-
     """
     argue parse command line args
     """
@@ -711,45 +829,18 @@ def argue():
         help=""" Input source, like "/home/a/vid.ts"
                                 or "udp://@235.35.3.5:3535"
                                 or "https://futzu.com/xaa.ts"
+                                [default: stdin]
+
                                 """,
     )
 
     parser.add_argument(
-        "-o",
-        "--output_dir",
-        default=".",
-        help="""Directory for segments and index.m3u8
-                ( created if it does not exist ) """,
-    )
-
-    parser.add_argument(
-        "-s",
-        "--sidecar_file",
-        default=None,
-        help="""Sidecar file of scte35 cues. each line contains  PTS, Cue""",
-    )
-
-    parser.add_argument(
-        "-t",
-        "--time",
-        default=2,
-        type=float,
-        help="Segment time in seconds ( default is 2)",
-    )
-
-    parser.add_argument(
-        "-T",
-        "--hls_tag",
-        default="x_cue",
-        help="x_scte35, x_cue, x_daterange, or x_splicepoint  (default x_cue)",
-    )
-
-    parser.add_argument(
-        "-w",
-        "--window_size",
-        default=5,
-        type=int,
-        help="sliding window size(default:5)",
+        "-c",
+        "--continue_m3u8",
+        action="store_const",
+        default=False,
+        const=True,
+        help="Resume writing index.m3u8 [default:False]",
     )
 
     parser.add_argument(
@@ -758,7 +849,7 @@ def argue():
         action="store_const",
         default=False,
         const=True,
-        help="delete segments ( enables --live )",
+        help="delete segments (enables --live) [default:False]",
     )
 
     parser.add_argument(
@@ -767,33 +858,24 @@ def argue():
         action="store_const",
         default=False,
         const=True,
-        help="Flag for a live event ( enables sliding window m3u8 )",
+        help="Flag for a live event (enables sliding window m3u8) [default:False]",
     )
 
     parser.add_argument(
-        "-r",
-        "--replay",
+        "-n",
+        "--no_discontinuity",
         action="store_const",
         default=False,
         const=True,
-        help="Flag for replay (looping) ( enables --live and --delete )",
+        help="Flag to disable adding #EXT-X-DISCONTINUITY tags at splice points [default:False]",
     )
 
     parser.add_argument(
-        "-S",
-        "--shulga",
-        action="store_const",
-        default=False,
-        const=True,
-        help="Flag to enable Shulga iframe detection mode",
-    )
-    parser.add_argument(
-        "-v",
-        "--version",
-        action="store_const",
-        default=False,
-        const=True,
-        help="Show version",
+        "-o",
+        "--output_dir",
+        default=".",
+        help="""Directory for segments and index.m3u8
+                (created if needed) [default:'.'] """,
     )
 
     parser.add_argument(
@@ -802,7 +884,66 @@ def argue():
         action="store_const",
         default=False,
         const=True,
-        help="Flag to add Program Date Time tags to index.m3u8 ( enables --live)",
+        help="Flag to add Program Date Time tags to index.m3u8 ( enables --live) [default:False]",
+    )
+
+    parser.add_argument(
+        "-r",
+        "--replay",
+        action="store_const",
+        default=False,
+        const=True,
+        help="""Flag for replay aka looping
+        (enables --live,--delete) [default:False]
+        """,
+    )
+
+    parser.add_argument(
+        "-s",
+        "--sidecar_file",
+        default=None,
+        help="""Sidecar file of SCTE-35 (pts,cue) pairs.[default:None]""",
+    )
+
+    parser.add_argument(
+        "-S",
+        "--shulga",
+        action="store_const",
+        default=False,
+        const=True,
+        help="Flag to enable Shulga iframe detection mode [default:False]",
+    )
+
+    parser.add_argument(
+        "-t",
+        "--time",
+        default=2,
+        type=float,
+        help="Segment time in seconds [default:2]",
+    )
+
+    parser.add_argument(
+        "-T",
+        "--hls_tag",
+        default="x_cue",
+        help="x_scte35, x_cue, x_daterange, or x_splicepoint [default:x_cue]",
+    )
+
+    parser.add_argument(
+        "-w",
+        "--window_size",
+        default=5,
+        type=int,
+        help="sliding window size (enables --live) [default:5]",
+    )
+
+    parser.add_argument(
+        "-v",
+        "--version",
+        action="store_const",
+        default=False,
+        const=True,
+        help="Show version",
     )
 
     return parser.parse_args()
@@ -818,10 +959,14 @@ def cli():
      cli()
 
     """
-    stuff = X9K3()
-    stuff.run()
+    args = argue()
+    x9 = X9K3()
+    x9.decode()
+    while args.replay:
+        x9 = X9K3()
+        x9.continue_m3u8()
+        x9.decode()
 
 
 if __name__ == "__main__":
-    x9k = X9K3()
-    x9k.run()
+    cli()
